@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: setup.py 11474 2019-04-20 18:48:58Z Lavender $
+# $Id: setup.py 11562 2019-05-17 19:18:19Z Lavender $
 #
 # Copyright (c) 2017 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: Lavender $
-# $Date: 2019-04-21 03:48:58 +0900 (週日, 21 四月 2019) $
-# $Revision: 11474 $
+# $Date: 2019-05-18 03:18:19 +0800 (週六, 18 五月 2019) $
+# $Revision: 11562 $
 
 import os
 import sys
@@ -23,7 +23,9 @@ import shutil
 import json
 import argparse
 import locale
+import hashlib
 import traceback
+import pkg_resources
 
 try:
     import colorama 
@@ -76,6 +78,43 @@ CKEDITOR_SETTINGS = {
     'skin': 'moono-lisa',
 }
 '''
+
+SCHEMA_ORIGIN_MD5 = "c27ea4f5a250178b0a417b719c40c3a8"
+SCHEMA_FIX_MD5 = "08fc81f9012c4a7f94782edc030885e7"
+
+SCHEMA_FIX = '''
+    def __enter__(self):
+        # Some SQLite schema alterations need foreign key constraints to be
+        # disabled. Enforce it here for the duration of the transaction.
+        self.connection.disable_constraint_checking()
+        self.connection.cursor().execute('PRAGMA legacy_alter_table = ON')
+        return super().__enter__()
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        super().__exit__(exc_type, exc_value, traceback)
+        self.connection.cursor().execute('PRAGMA legacy_alter_table = OFF')
+        self.connection.enable_constraint_checking()
+'''
+
+SCHEMA_ORIGIN = '''
+    def __enter__(self):
+        with self.connection.cursor() as c:
+            # Some SQLite schema alterations need foreign key constraints to be
+            # disabled. This is the default in SQLite but can be changed with a
+            # build flag and might change in future, so can't be relied upon.
+            # We enforce it here for the duration of the transaction.
+            c.execute('PRAGMA foreign_keys')
+            self._initial_pragma_fk = c.fetchone()[0]
+            c.execute('PRAGMA foreign_keys = 0')
+        return super(DatabaseSchemaEditor, self).__enter__()
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        super(DatabaseSchemaEditor, self).__exit__(exc_type, exc_value, traceback)
+        with self.connection.cursor() as c:
+            # Restore initial FK setting - PRAGMA values can't be parametrized
+            c.execute('PRAGMA foreign_keys = %s' % int(self._initial_pragma_fk))
+'''
+
 def copyToPath(baseDir, templateDir, staticDir, mode):
     if not templateDir:
         templateDir = os.path.join(baseDir, 'templates')
@@ -292,6 +331,32 @@ def loadData(baseDir, language):
     modifyData()
        
     return True
+    
+def fixSchema(schemaPath):
+    try:
+        with open(schemaPath, 'r') as f:
+            content = f.read()
+        
+        if hashlib.md5(content.encode()).hexdigest() == SCHEMA_ORIGIN_MD5:
+            content = content.replace(SCHEMA_ORIGIN, SCHEMA_FIX)
+            
+            with open(schemaPath, 'w') as f:
+                f.write(content)
+    except Exception as e:
+        pass
+    
+def revertSchema(schemaPath):
+    try:
+        with open(schemaPath, 'r') as f:
+            content = f.read()
+            
+        if hashlib.md5(content.encode()).hexdigest() == SCHEMA_FIX_MD5:
+            content = content.replace(SCHEMA_FIX, SCHEMA_ORIGIN)
+            
+            with open(schemaPath, 'w') as f:
+                f.write(content)
+    except Exception as e:
+        pass
        
 if __name__ == "__main__":
     try:
@@ -308,7 +373,15 @@ if __name__ == "__main__":
             help="Run the configuration wizard")   
        
         args = parser.parse_args()
-       
+        
+        # modify django schema.py
+        # issue: https://github.com/django/django/pull/10733/commits/c8ffdbe514b55ff5c9a2b8cb8bbdf2d3978c188f
+        try:
+            schemaPath = pkg_resources.resource_filename(
+                'django.db.backends.sqlite3', 'schema.py')
+        except Exception as e:
+            schemaPath = None
+         
         wizard = args.wizard
         mode = 'user'
         
@@ -355,6 +428,22 @@ if __name__ == "__main__":
             
             result = os.system(
                 "pip install djangocms-installer==%s" % installVersion)
+                
+            try:
+                import django
+            except Exception as e:
+                os.system("pip install django==1.11.20")
+                
+            if schemaPath:
+                # modify django schema.py
+                logger.warning(colored(
+                    "There is a django issue (ticket: 29182) which is "
+                    "schema corruption issue on SQLite 3.26+. We are trying "
+                    "to fix this issue for you to patch your code..."
+                    "(You can see https://github.com/django/django/pull"
+                    "/10733/commits/c8ffdbe514b55ff5c9a2b8cb8bbdf2d3978c188f"
+                    " for more details)", "yellow"))
+                fixSchema(schemaPath)
             
             if result != 0:
                 logger.error(colored(
@@ -501,10 +590,16 @@ if __name__ == "__main__":
         os.chdir(baseDir)
            
         success = loadData(baseDir, settings.LANGUAGE_CODE)
+        
+        if schemaPath:
+            # revert django schema.py
+            revertSchema(schemaPath)
+        
         if not success:
             print("")
             logger.error(
                 colored("Please check the project's db was migrated.", "red"))
+            raise
         else:
             attrs=['bold']
             print("")
